@@ -13,6 +13,8 @@
 #include <zypp-curl/private/curlhelper_p.h>
 #include <zypp-core/fs/PathInfo.h>
 
+#include <utility>
+
 #include "basicdownloader_p.h"
 
 namespace zyppng {
@@ -79,7 +81,7 @@ namespace zyppng {
     const auto &spec = sm._spec;
 
     _request = std::make_shared<Request>( ::internal::clearQueryString(url), spec.targetPath() ) ;
-    _request->_myMirror = mirror;
+    _request->_myMirror = std::move(mirror);
     _request->_originalUrl = url;
     _request->transferSettings() = set;
 
@@ -107,10 +109,10 @@ namespace zyppng {
       return failed("Request was not intialized before starting it.");
 
     if ( _chksumtype  && _chksumVec ) {
-      std::shared_ptr<zypp::Digest> fileDigest = std::make_shared<zypp::Digest>();
+      std::optional<zypp::Digest> fileDigest = zypp::Digest();
       if ( fileDigest->create( *_chksumtype ) )
         // to run the checksum for the full file we need to request one big range with open end
-        _request->addRequestRange( 0, 0, fileDigest, *_chksumVec );
+        _request->addRequestRange( 0, 0, std::move(fileDigest), *_chksumVec );
     }
 
     if ( sm._spec.checkExistsOnly() )
@@ -167,7 +169,7 @@ namespace zyppng {
   {
     auto &sm = stateMachine();
     const off_t expFSize = sm._spec.expectedFileSize();
-    if ( expFSize  > 0 && expFSize < dlnow ) {
+    if ( expFSize  > 0 && expFSize < req.downloadedByteCount() ) {
       sm._requestDispatcher->cancel( req, NetworkRequestErrorPrivate::customError( NetworkRequestError::ExceededMaxLen ) );
       return;
     }
