@@ -15,6 +15,7 @@
 #include <iostream>
 #include <fstream>
 
+#include <utility>
 #include <zypp-media/MediaConfig>
 #include <zypp-core/base/Function.h>
 #include <zypp-core/base/Logger.h>
@@ -83,7 +84,12 @@ namespace zypp
   //
   struct CredentialManager::Impl
   {
-    Impl(const CredManagerOptions & options);
+    Impl(CredManagerOptions &&options);
+
+    Impl(const Impl &) = delete;
+    Impl(Impl &&) = delete;
+    Impl &operator=(const Impl &) = delete;
+    Impl &operator=(Impl &&) = delete;
 
     ~Impl()
     {}
@@ -117,8 +123,8 @@ namespace zypp
   //
   //////////////////////////////////////////////////////////////////////
 
-  CredentialManager::Impl::Impl(const CredManagerOptions & options)
-    : _options(options)
+  CredentialManager::Impl::Impl(CredManagerOptions &&options)
+    : _options(std::move(options))
     , _globalDirty(false)
     , _userDirty(false)
   {
@@ -179,7 +185,7 @@ namespace zypp
   }
 
 
-  static AuthData_Ptr findIn(const CredentialManager::CredentialSet & set,
+  AuthData_Ptr CredentialManager::findIn(const CredentialManager::CredentialSet & set,
                              const Url & url,
                              url::ViewOption vopt)
   {
@@ -200,7 +206,6 @@ namespace zypp
     return AuthData_Ptr();
   }
 
-
   AuthData_Ptr CredentialManager::Impl::getCred(const Url & url) const
   {
     AuthData_Ptr result;
@@ -267,13 +272,15 @@ namespace zypp
   }
 
   static int save_creds_in_file(
-      const CredentialManager::CredentialSet creds,
+      CredentialManager::CredentialSet &creds,
       const Pathname & file,
       const mode_t mode)
   {
     int ret = 0;
     filesystem::assert_file_mode( file, mode );
 
+    const auto now = time( nullptr );
+
     PathInfo pi { file };
     if ( pi.userMayRW() ) try {
       // make sure only our thread accesses the file
@@ -284,7 +291,7 @@ namespace zypp
       for_(it, creds.begin(), creds.end())
       {
         (*it)->dumpAsIniOn(fs);
-        (*it)->setLastDatabaseUpdate( time( nullptr ) );
+        (*it)->setLastDatabaseUpdate( now );
         fs << endl;
       }
       if ( !fs ) {
@@ -318,8 +325,8 @@ namespace zypp
   //
   //////////////////////////////////////////////////////////////////////
 
-  CredentialManager::CredentialManager(const CredManagerOptions & opts)
-    : _pimpl(new Impl(opts))
+  CredentialManager::CredentialManager(CredManagerOptions opts)
+    : _pimpl(new Impl(std::move(opts)))
   {}
 
 
@@ -349,6 +356,22 @@ namespace zypp
       saveInFile(cred, credfile);
   }
 
+  time_t CredentialManager::timestampForCredDatabase ( const zypp::Url &url )
+  {
+    Pathname credfile;
+    if ( url.isValid() ) {
+      credfile = url.getQueryParam("credentials");
+    }
+
+    if (credfile.empty())
+      credfile = _pimpl->_options.userCredFilePath;
+
+    zypp::PathInfo pi(credfile);
+    if ( pi.isExist() && pi.isFile() )
+      return pi.mtime();
+
+    return 0;
+  }
 
   void CredentialManager::addGlobalCred(const AuthData & cred)
   {
@@ -421,7 +444,7 @@ namespace zypp
     CredentialManager::CredentialSet creds;
     creds.insert(c_ptr);
 
-    int ret;
+    int ret = 0;
     if (credFile.absolute())
       ret = save_creds_in_file(creds, credFile, 0640);
     else
