@@ -17,7 +17,7 @@
 #include <zypp/base/ReferenceCounted.h>
 #include <zypp/base/NonCopyable.h>
 #include <zypp/base/PtrTypes.h>
-#include <zypp/APIConfig.h>
+#include <zypp-core/Globals.h>
 
 #include <zypp/Product.h>
 #include <zypp/Pathname.h>
