@@ -61,6 +61,76 @@ namespace zypp
   namespace
   { /////////////////////////////////////////////////////////////////
 
+    ///////////////////////////////////////////////////////////////////
+    // From rpm's lib/rpmrc.c
+#   if defined(__linux__) && defined(__x86_64__)
+    static inline void cpuid(uint32_t op, uint32_t op2, uint32_t *eax, uint32_t *ebx, uint32_t *ecx, uint32_t *edx)
+    {
+        asm volatile (
+            "cpuid\n"
+        : "=a" (*eax), "=b" (*ebx), "=c" (*ecx), "=d" (*edx)
+        : "a" (op), "c" (op2));
+    }
+
+    /* From gcc's gcc/config/i386/cpuid.h */
+    /* Features (%eax == 1) */
+    /* %ecx */
+    #define bit_SSE3        (1 << 0)
+    #define bit_SSSE3       (1 << 9)
+    #define bit_FMA         (1 << 12)
+    #define bit_CMPXCHG16B  (1 << 13)
+    #define bit_SSE4_1      (1 << 19)
+    #define bit_SSE4_2      (1 << 20)
+    #define bit_MOVBE       (1 << 22)
+    #define bit_POPCNT      (1 << 23)
+    #define bit_OSXSAVE     (1 << 27)
+    #define bit_AVX         (1 << 28)
+    #define bit_F16C        (1 << 29)
+
+    /* Extended Features (%eax == 0x80000001) */
+    /* %ecx */
+    #define bit_LAHF_LM     (1 << 0)
+    #define bit_LZCNT       (1 << 5)
+
+    /* Extended Features (%eax == 7) */
+    /* %ebx */
+    #define bit_BMI         (1 << 3)
+    #define bit_AVX2        (1 << 5)
+    #define bit_BMI2        (1 << 8)
+    #define bit_AVX512F     (1 << 16)
+    #define bit_AVX512DQ    (1 << 17)
+    #define bit_AVX512CD    (1 << 28)
+    #define bit_AVX512BW    (1 << 30)
+    #define bit_AVX512VL    (1u << 31)
+
+    static int get_x86_64_level(void)
+    {
+        int level = 1;
+
+        unsigned int op_1_ecx = 0, op_80000001_ecx = 0, op_7_ebx = 0, unused = 0;
+        cpuid(1, 0, &unused, &unused, &op_1_ecx, &unused);
+        cpuid(0x80000001, 0, &unused, &unused, &op_80000001_ecx, &unused);
+        cpuid(7, 0, &unused, &op_7_ebx, &unused, &unused);
+
+        const unsigned int op_1_ecx_lv2 = bit_SSE3 | bit_SSSE3 | bit_CMPXCHG16B | bit_SSE4_1 | bit_SSE4_2 | bit_POPCNT;
+        if ((op_1_ecx & op_1_ecx_lv2) == op_1_ecx_lv2 && (op_80000001_ecx & bit_LAHF_LM))
+            level = 2;
+
+        const unsigned int op_1_ecx_lv3 = bit_FMA | bit_MOVBE | bit_OSXSAVE | bit_AVX | bit_F16C;
+        const unsigned int op_7_ebx_lv3 = bit_BMI | bit_AVX2 | bit_BMI2;
+        if (level == 2 && (op_1_ecx & op_1_ecx_lv3) == op_1_ecx_lv3 && (op_7_ebx & op_7_ebx_lv3) == op_7_ebx_lv3
+            && (op_80000001_ecx & bit_LZCNT))
+            level = 3;
+
+        const unsigned int op_7_ebx_lv4 = bit_AVX512F | bit_AVX512DQ | bit_AVX512CD | bit_AVX512BW | bit_AVX512VL;
+        if (level == 3 && (op_7_ebx & op_7_ebx_lv4) == op_7_ebx_lv4)
+            level = 4;
+
+        return level;
+    }
+#   endif
+    ///////////////////////////////////////////////////////////////////
+
     /** Determine system architecture evaluating \c uname and \c /proc/cpuinfo.
     */
     Arch _autodetectSystemArchitecture()
@@ -75,7 +145,27 @@ namespace zypp
       Arch architecture( buf.machine );
       MIL << "Uname architecture is '" << buf.machine << "'" << endl;
 
-      if ( architecture == Arch_i686 )
+      if ( architecture == Arch_x86_64 )
+      {
+#if     defined(__linux__) && defined(__x86_64__)
+        switch ( get_x86_64_level() )
+        {
+          case 2:
+            architecture = Arch_x86_64_v2;
+            WAR << "CPU has 'x86_64': architecture upgraded to '" << architecture << "'" << endl;
+            break;
+          case 3:
+            architecture = Arch_x86_64_v3;
+            WAR << "CPU has 'x86_64': architecture upgraded to '" << architecture << "'" << endl;
+            break;
+          case 4:
+            architecture = Arch_x86_64_v4;
+            WAR << "CPU has 'x86_64': architecture upgraded to '" << architecture << "'" << endl;
+            break;
+        }
+#       endif
+      }
+      else if ( architecture == Arch_i686 )
       {
         // some CPUs report i686 but dont implement cx8 and cmov
         // check for both flags in /proc/cpuinfo and downgrade
@@ -168,7 +258,7 @@ namespace zypp
       else if ( architecture == Arch_ppc64 )
       {
         const char * platform = (const char *)getauxval( AT_PLATFORM );
-        int powerlvl;
+        int powerlvl = 0;
         if ( platform && sscanf( platform, "power%d", &powerlvl ) == 1 && powerlvl > 6 )
           architecture = Arch_ppc64p7;
       }
@@ -242,7 +332,7 @@ namespace zypp
   template<class Tp>
       struct Option
       {
-        typedef Tp value_type;
+        using value_type = Tp;
 
         /** No default ctor, explicit initialisation! */
         Option( value_type initial_r )
@@ -272,8 +362,8 @@ namespace zypp
   template<class Tp>
       struct DefaultOption : public Option<Tp>
       {
-        typedef Tp         value_type;
-        typedef Option<Tp> option_type;
+        using value_type = Tp;
+        using option_type = Option<Tp>;
 
         explicit DefaultOption( value_type initial_r )
         : Option<Tp>( initial_r )
@@ -314,7 +404,7 @@ namespace zypp
   */
   class ZConfig::Impl
   {
-    typedef std::set<std::string> MultiversionSpec;
+    using MultiversionSpec = std::set<std::string>;
 
     /// Settings that follow a changed Target
     struct TargetDefaults
@@ -415,6 +505,8 @@ namespace zypp
         , pkgGpgCheck			( indeterminate )
         , apply_locks_file		( true )
         , pluginsPath			( "/usr/lib/zypp/plugins" )
+        , geoipEnabled ( true )
+        , geoipHosts { "download.opensuse.org" }
       {
         MIL << "libzypp: " LIBZYPP_VERSION_STRING << endl;
         if ( PathInfo(_parsedZyppConf).isExist() )
@@ -424,7 +516,7 @@ namespace zypp
                 sit != dict.sectionsEnd();
                 ++sit )
           {
-            std::string section(*sit);
+            const std::string& section(*sit);
             //MIL << section << endl;
             for ( IniDict::entry_const_iterator it = dict.entriesBegin(*sit);
                   it != dict.entriesEnd(*sit);
@@ -518,6 +610,9 @@ namespace zypp
                 {
                   download_mediaMountdir.restoreToDefault( Pathname(value) );
                 }
+                else if ( entry == "download.use_geoip_mirror") {
+                  geoipEnabled = str::strToBool( value, geoipEnabled );
+                }
                 else if ( entry == "commit.downloadMode" )
                 {
                   commit_downloadMode.set( deserializeDownloadMode( value ) );
@@ -569,15 +664,18 @@ namespace zypp
                 }
                 else if ( entry == "update.datadir" )
                 {
-                  update_data_path = Pathname(value);
+                  // ignore, this is a constant anyway and should not be user configurabe
+                  // update_data_path = Pathname(value);
                 }
                 else if ( entry == "update.scriptsdir" )
                 {
-                  update_scripts_path = Pathname(value);
+                  // ignore, this is a constant anyway and should not be user configurabe
+                  // update_scripts_path = Pathname(value);
                 }
                 else if ( entry == "update.messagessdir" )
                 {
-                  update_messages_path = Pathname(value);
+                  // ignore, this is a constant anyway and should not be user configurabe
+                  // update_messages_path = Pathname(value);
                 }
                 else if ( entry == "update.messages.notify" )
                 {
@@ -625,8 +723,11 @@ namespace zypp
         MIL << "ZConfig singleton created." << endl;
       }
 
-      ~Impl()
-      {}
+      Impl(const Impl &) = delete;
+      Impl(Impl &&) = delete;
+      Impl &operator=(const Impl &) = delete;
+      Impl &operator=(Impl &&) = delete;
+      ~Impl() {}
 
       void notifyTargetChanged()
       {
@@ -675,9 +776,6 @@ namespace zypp
     std::string cfg_kernel_keep_spec;
     Pathname locks_file;
 
-    Pathname update_data_path;
-    Pathname update_scripts_path;
-    Pathname update_messages_path;
     DefaultOption<std::string> updateMessagesNotify;
 
     bool	repo_add_probe;
@@ -712,6 +810,10 @@ namespace zypp
 
     Option<Pathname> pluginsPath;
 
+    bool geoipEnabled;
+
+    std::vector<std::string> geoipHosts;
+
     /* Other config singleton instances */
     MediaConfig &_mediaConf = MediaConfig::instance();
 
@@ -731,7 +833,7 @@ namespace zypp
     // overlay all system specific values.
     struct MultiversionMap
     {
-      typedef std::map<Pathname,MultiversionSpec> SpecMap;
+      using SpecMap = std::map<Pathname, MultiversionSpec>;
 
       MultiversionSpec & getSpec( Pathname root_r, const Impl & zConfImpl_r )	// from system at root
       {
@@ -764,7 +866,7 @@ namespace zypp
       {	return _specMap[Pathname()]; }
 
     private:
-      int scanConfAt( const Pathname root_r, MultiversionSpec & spec_r, const Impl & zConfImpl_r )
+      int scanConfAt( const Pathname& root_r, MultiversionSpec & spec_r, const Impl & zConfImpl_r )
       {
         static const str::regex rx( "^multiversion *= *(.*)" );
         str::smatch what;
@@ -780,7 +882,7 @@ namespace zypp
                                 } );
       }
 
-      void scanDirAt( const Pathname root_r, MultiversionSpec & spec_r, const Impl & zConfImpl_r )
+      void scanDirAt( const Pathname& root_r, MultiversionSpec & spec_r, const Impl & zConfImpl_r )
       {
         // NOTE:  Actually we'd need to scan and use the root_r! zypp.conf values.
         Pathname multiversionDir( zConfImpl_r.cfg_multiversion_path );
@@ -1024,6 +1126,18 @@ namespace zypp
   Pathname ZConfig::needrebootPath() const
   { return configPath()/"needreboot.d"; }
 
+  void ZConfig::setGeoipEnabled( bool enable )
+  { _pimpl->geoipEnabled = enable; }
+
+  bool ZConfig::geoipEnabled () const
+  { return _pimpl->geoipEnabled; }
+
+  Pathname ZConfig::geoipCachePath() const
+  { return builtinRepoCachePath()/"geoip.d"; }
+
+  const std::vector<std::string> ZConfig::geoipHostnames () const
+  { return _pimpl->geoipHosts; }
+
   Pathname ZConfig::varsPath() const
   {
     return ( _pimpl->cfg_vars_path.empty()
@@ -1151,22 +1265,28 @@ namespace zypp
   bool ZConfig::apply_locks_file() const
   { return _pimpl->apply_locks_file; }
 
-  Pathname ZConfig::update_dataPath() const
+  Pathname ZConfig::update_dataPath()
+#if LEGACY(1735)
+  const
+#endif
   {
-    return ( _pimpl->update_data_path.empty()
-        ? Pathname("/var/adm") : _pimpl->update_data_path );
+    return Pathname("/var/adm");
   }
 
-  Pathname ZConfig::update_messagesPath() const
+  Pathname ZConfig::update_messagesPath()
+#if LEGACY(1735)
+  const
+#endif
   {
-    return ( _pimpl->update_messages_path.empty()
-             ? Pathname(update_dataPath()/"update-messages") : _pimpl->update_messages_path );
+    return Pathname(update_dataPath()/"update-messages");
   }
 
-  Pathname ZConfig::update_scriptsPath() const
+  Pathname ZConfig::update_scriptsPath()
+#if LEGACY(1735)
+  const
+#endif
   {
-    return ( _pimpl->update_scripts_path.empty()
-             ? Pathname(update_dataPath()/"update-scripts") : _pimpl->update_scripts_path );
+    return Pathname(update_dataPath()/"update-scripts");
   }
 
   std::string ZConfig::updateMessagesNotify() const
