@@ -12,9 +12,11 @@
 #include "librpm.h"
 
 #include <iostream>
+#include <utility>
 
 #include <zypp/base/Logger.h>
 #include <zypp/PathInfo.h>
+#include <zypp-core/AutoDispose.h>
 #include <zypp/target/rpm/librpmDb.h>
 #include <zypp/target/rpm/RpmHeader.h>
 #include <zypp/target/rpm/RpmException.h>
@@ -30,6 +32,56 @@ namespace target
 {
 namespace rpm
 {
+  namespace internal
+  {
+    // helper functions here expect basic checks on arguments have been performed.
+    // (globalInit done, root is absolute, dbpath not empty, ...)
+    inline const Pathname & rpmDefaultDbPath()
+    {
+      static const Pathname _val = [](){
+        Pathname ret = librpmDb::expand( "%{_dbpath}" );
+        if ( ret.empty() ) {
+          ret = "/usr/lib/sysimage/rpm";
+          WAR << "Looks like rpm has no %{_dbpath} set!?! Assuming " << ret << endl;
+        }
+        return ret;
+      }();
+      return _val;
+    }
+
+    inline Pathname suggestedDbPath( const Pathname & root_r )
+    {
+      if ( PathInfo( root_r ).isDir() ) {
+        // If a known dbpath exsists, we continue to use it
+        for ( auto p : { "/var/lib/rpm", "/usr/lib/sysimage/rpm" } ) {
+          if ( PathInfo( root_r/p, PathInfo::LSTAT/*!no symlink*/ ).isDir() ) {
+            MIL << "Suggest existing database at " << dumpPath( root_r, p ) << endl;
+            return p;
+          }
+        }
+      }
+      const Pathname & defaultDbPath { rpmDefaultDbPath() };
+      MIL << "Suggest rpm dbpath " << dumpPath( root_r, defaultDbPath ) << endl;
+      return defaultDbPath;
+    }
+
+    inline Pathname sanitizedDbPath( const Pathname & root_r, const Pathname & dbPath_r )
+    { return dbPath_r.empty() ? suggestedDbPath( root_r ) : dbPath_r; }
+
+    inline bool dbExists( const Pathname & root_r, const Pathname & dbPath_r )
+    {
+      Pathname dbdir { root_r / sanitizedDbPath( root_r, dbPath_r ) };
+      return PathInfo(dbdir).isDir() && ( PathInfo(dbdir/"Packages").isFile() || PathInfo(dbdir/"Packages.db").isFile() );
+    }
+
+  } // namespace internal
+
+///////////////////////////////////////////////////////////////////
+//
+//	CLASS NAME : librpmDb (ststic interface)
+//
+///////////////////////////////////////////////////////////////////
+
 ///////////////////////////////////////////////////////////////////
 //
 //	CLASS NAME : librpmDb::D
@@ -38,80 +90,51 @@ namespace rpm
  **/
 class librpmDb::D
 {
-  D & operator=( const D & ); // NO ASSIGNMENT!
-  D ( const D & );            // NO COPY!
+  D ( const D & ) = delete;            // NO COPY!
+  D & operator=( const D & ) = delete; // NO ASSIGNMENT!
+  D(D &&) = delete;
+  D &operator=(D &&) = delete;
 public:
 
   const Pathname _root;   // root directory for all operations
   const Pathname _dbPath; // directory (below root) that contains the rpmdb
-  rpmts _ts;              // transaction handle, includes database
-  shared_ptr<RpmException> _error;  // database error
+  AutoDispose<rpmts> _ts; // transaction handle, includes database
 
   friend std::ostream & operator<<( std::ostream & str, const D & obj )
-  {
-    str << "{" << obj._error  << "(" << obj._root << ")" << obj._dbPath << "}";
-    return str;
-  }
+  { return str << "{" << dumpPath( obj._root, obj._dbPath ) << "}"; }
 
-  D( const Pathname & root_r, const Pathname & dbPath_r, bool readonly_r )
-      : _root  ( root_r )
-      , _dbPath( dbPath_r )
-      , _ts    ( 0 )
+  D( Pathname root_r, Pathname dbPath_r, bool readonly_r )
+  : _root   { std::move(root_r) }
+  , _dbPath { std::move(dbPath_r) }
+  , _ts     { nullptr }
   {
-    _error.reset();
-    // set %_dbpath macro
-    ::addMacro( NULL, "_dbpath", NULL, _dbPath.asString().c_str(), RMIL_CMDLINE );
-
-    _ts = ::rpmtsCreate();
+    _ts = AutoDispose<rpmts>( ::rpmtsCreate(), ::rpmtsFree );
     ::rpmtsSetRootDir( _ts, _root.c_str() );
 
     // open database (creates a missing one on the fly)
-    int res = ::rpmtsOpenDB( _ts, (readonly_r ? O_RDONLY : O_RDWR ));
-    if ( res )
-    {
+    OnScopeExit cleanup;
+    if ( _dbPath != internal::rpmDefaultDbPath() ) {
+      // temp set %{_dbpath} macro for rpmtsOpenDB
+      cleanup.setDispose( &macroResetDbpath );
+      macroSetDbpath( _dbPath );
+    }
+    int res = ::rpmtsOpenDB( _ts, (readonly_r ? O_RDONLY : O_RDWR ) );
+    if ( res ) {
       ERR << "rpmdbOpen error(" << res << "): " << *this << endl;
-      _error = shared_ptr<RpmDbOpenException>(new RpmDbOpenException(_root, _dbPath));
-      rpmtsFree(_ts);
-      ZYPP_THROW(*_error);
-      return;
-    }
-
-    DBG << "DBACCESS " << *this << endl;
-  }
-
-  ~D()
-  {
-    if ( _ts )
-    {
-      ::rpmtsFree(_ts);
+      ZYPP_THROW(RpmDbOpenException(_root, _dbPath));
     }
   }
+
+private:
+  static void macroSetDbpath( const Pathname & dppath_r )
+  { ::addMacro( NULL, "_dbpath", NULL, dppath_r.asString().c_str(), RMIL_CMDLINE ); }
+
+  static void macroResetDbpath()
+  { macroSetDbpath( internal::rpmDefaultDbPath() ); }
 };
 
 ///////////////////////////////////////////////////////////////////
 
-///////////////////////////////////////////////////////////////////
-//
-//	CLASS NAME : librpmDb (ststic interface)
-//
-///////////////////////////////////////////////////////////////////
-
-Pathname         librpmDb::_defaultRoot  ( "/" );
-// NOTE: Former variable, but now locked to "/var/lib/rpm".
-// A custom dbPath is not actually needed and would only work
-// reliably if libsolv also supports it.
-// The protected librpmDb ctor would allow to build a db_const_iterator
-// to access (ro) a database at a custom location.
-const Pathname   librpmDb::_defaultDbPath( "/var/lib/rpm" );
-librpmDb::constPtr librpmDb::_defaultDb;
-bool             librpmDb::_dbBlocked    ( true );
-
-///////////////////////////////////////////////////////////////////
-//
-//
-//	METHOD NAME : librpmDb::globalInit
-//	METHOD TYPE : bool
-//
 bool librpmDb::globalInit()
 {
   static bool initialized = false;
@@ -126,210 +149,72 @@ bool librpmDb::globalInit()
     return false;
   }
 
-  initialized = true; // Necessary to be able to use exand().
-
-#define OUTVAL(n) << " (" #n ":" << expand( "%{" #n "}" ) << ")"
-  MIL << "librpm init done:"
-  OUTVAL(_target)
-  OUTVAL(_dbpath)
-  << endl;
-#undef OUTVAL
+  initialized = true; // Necessary to be able to use exand() in rpmDefaultDbPath().
+  const Pathname & rpmDefaultDbPath { internal::rpmDefaultDbPath() };  // init rpmDefaultDbPath()!
+  MIL << "librpm init done: (_target:" << expand( "%{_target}" ) << ") (_dbpath:" << rpmDefaultDbPath << ")" << endl;
   return initialized;
 }
 
-///////////////////////////////////////////////////////////////////
-//
-//
-//	METHOD NAME : librpmDb::expand
-//	METHOD TYPE : std::string
-//
 std::string librpmDb::expand( const std::string & macro_r )
 {
   if ( ! globalInit() )
     return macro_r;  // unexpanded
 
-  char * val = ::rpmExpand( macro_r.c_str(), NULL );
-  if ( !val )
-    return "";
+  AutoFREE<char> val = ::rpmExpand( macro_r.c_str(), NULL );
+  if ( val )
+    return std::string( val );
 
-  std::string ret( val );
-  free( val );
-  return ret;
+  return std::string();
 }
 
-///////////////////////////////////////////////////////////////////
-//
-//
-//	METHOD NAME : librpmDb::newLibrpmDb
-//	METHOD TYPE : librpmDb *
-//
-librpmDb * librpmDb::newLibrpmDb()
+Pathname librpmDb::suggestedDbPath( const Pathname & root_r )
 {
-  // initialize librpm
+  if ( ! root_r.absolute() )
+    ZYPP_THROW(RpmInvalidRootException( root_r, "" ));
+
+  // initialize librpm (for rpmDefaultDbPath)
   if ( ! globalInit() )
-  {
     ZYPP_THROW(GlobalRpmInitException());
-  }
 
-  // open rpmdb
-  librpmDb * ret = 0;
-  try
-  {
-    ret = new librpmDb( _defaultRoot, _defaultDbPath, /*readonly*/true );
-  }
-  catch (const RpmException & excpt_r)
-  {
-    ZYPP_CAUGHT(excpt_r);
-    delete ret;
-    ret = 0;
-    ZYPP_RETHROW(excpt_r);
-  }
-  return ret;
+  return internal::suggestedDbPath( root_r );
 }
 
-///////////////////////////////////////////////////////////////////
-//
-//
-//	METHOD NAME : librpmDb::dbAccess
-//	METHOD TYPE : PMError
-//
-void librpmDb::dbAccess( const Pathname & root_r )
+bool librpmDb::dbExists( const Pathname & root_r, const Pathname & dbPath_r )
 {
-  if ( _defaultDb )
-  {
-    // already accessing a database: switching is not allowed.
-    if ( _defaultRoot == root_r )
-      return;
-    else
-      ZYPP_THROW(RpmDbAlreadyOpenException(_defaultRoot, _defaultDbPath, root_r, _defaultDbPath));
-  }
+  if ( ! root_r.absolute() )
+    ZYPP_THROW(RpmInvalidRootException( root_r, "" ));
 
-  // got no database: we could switch to a new one (even if blocked!)
+  // initialize librpm (for rpmDefaultDbPath)
+  if ( ! globalInit() )
+    ZYPP_THROW(GlobalRpmInitException());
 
-  if ( root_r.empty() || ! root_r.absolute() )
-    ZYPP_THROW(RpmInvalidRootException(root_r, _defaultDbPath));
-
-  PathInfo pi { root_r / _defaultDbPath };
-  if ( pi.isExist() && ! pi.isDir() ) {
-    RpmInvalidRootException excpt { root_r, _defaultDbPath };
-    excpt.addHistory( str::Str() << pi );
-    ZYPP_THROW(excpt);
-  }
-
-  _defaultRoot = root_r;
-  MIL << "Set new database location: " << stringPath( _defaultRoot, _defaultDbPath ) << endl;
-
-  return dbAccess();
+  return internal::dbExists( root_r, dbPath_r );
 }
 
-///////////////////////////////////////////////////////////////////
-//
-//
-//	METHOD NAME : librpmDb::dbAccess
-//	METHOD TYPE : PMError
-//
-void librpmDb::dbAccess()
+librpmDb::constPtr librpmDb::dbOpenIf( const Pathname & root_r, const Pathname & dbPath_r )
+{ return dbAccess( root_r, dbPath_r, /*create_r*/false ); }
+
+librpmDb::constPtr librpmDb::dbOpenCreate( const Pathname & root_r, const Pathname & dbPath_r )
+{ return dbAccess( root_r, dbPath_r, /*create_r*/true ); }
+
+librpmDb::constPtr librpmDb::dbAccess( const Pathname & root_r, const Pathname & dbPath_rx, bool create_r )
 {
-  if ( _dbBlocked )
-  {
-    ZYPP_THROW(RpmAccessBlockedException(_defaultRoot, _defaultDbPath));
+  if ( ! root_r.absolute() )
+    ZYPP_THROW(RpmInvalidRootException( root_r, "" ));
+
+  // initialize librpm (for rpmDefaultDbPath)
+  if ( ! globalInit() )
+    ZYPP_THROW(GlobalRpmInitException());
+
+  Pathname dbPath { internal::sanitizedDbPath( root_r, dbPath_rx ) };
+  bool dbExists = internal::dbExists( root_r, dbPath );
+
+  if ( not create_r && not dbExists ) {
+    WAR << "NoOpen not existing database " << dumpPath( root_r, dbPath ) << endl;
+    return nullptr;
   }
-
-  if ( !_defaultDb )
-  {
-    // get access
-    _defaultDb = newLibrpmDb();
-  }
-}
-
-///////////////////////////////////////////////////////////////////
-//
-//
-//	METHOD NAME : librpmDb::dbAccess
-//	METHOD TYPE : PMError
-//
-void librpmDb::dbAccess( librpmDb::constPtr & ptr_r )
-{
-  ptr_r = nullptr;
-  dbAccess();
-  ptr_r = _defaultDb;
-}
-
-///////////////////////////////////////////////////////////////////
-//
-//
-//	METHOD NAME : librpmDb::dbRelease
-//	METHOD TYPE : unsigned
-//
-unsigned librpmDb::dbRelease( bool force_r )
-{
-  if ( !_defaultDb )
-  {
-    return 0;
-  }
-
-  unsigned outstanding = _defaultDb->refCount() - 1; // refCount can't be 0
-
-  switch ( outstanding )
-  {
-  default:
-    if ( !force_r )
-    {
-      DBG << "dbRelease: keep access, outstanding " << outstanding << endl;
-      break;
-    }
-    // else fall through:
-  case 0:
-    DBG << "dbRelease: release" << (force_r && outstanding ? "(forced)" : "")
-    << ", outstanding " << outstanding << endl;
-
-    _defaultDb->_d._error = shared_ptr<RpmAccessBlockedException>(new RpmAccessBlockedException(_defaultDb->_d._root, _defaultDb->_d._dbPath));
-    // tag handle invalid
-    _defaultDb = 0;
-    break;
-  }
-
-  return outstanding;
-}
-
-///////////////////////////////////////////////////////////////////
-//
-//
-//	METHOD NAME : librpmDb::blockAccess
-//	METHOD TYPE : unsigned
-//
-unsigned librpmDb::blockAccess()
-{
-  MIL << "Block access" << endl;
-  _dbBlocked = true;
-  return dbRelease( /*force*/true );
-}
-
-///////////////////////////////////////////////////////////////////
-//
-//
-//	METHOD NAME : librpmDb::unblockAccess
-//	METHOD TYPE : void
-//
-void librpmDb::unblockAccess()
-{
-  MIL << "Unblock access" << endl;
-  _dbBlocked = false;
-}
-
-///////////////////////////////////////////////////////////////////
-//
-//
-//	METHOD NAME : librpmDb::dumpState
-//	METHOD TYPE : ostream &
-//
-std::ostream & librpmDb::dumpState( std::ostream & str )
-{
-  if ( !_defaultDb )
-  {
-    return str << "[librpmDb " << (_dbBlocked?"BLOCKED":"CLOSED") << " " << stringPath( _defaultRoot, _defaultDbPath ) << "]";
-  }
-  return str << "[" << _defaultDb << "]";
+  MIL << (dbExists?"Open":"Create") << " database " << dumpPath( root_r, dbPath ) << endl;
+  return new librpmDb( root_r, dbPath, /*readonly*/true );
 }
 
 ///////////////////////////////////////////////////////////////////
@@ -338,180 +223,60 @@ std::ostream & librpmDb::dumpState( std::ostream & str )
 //
 ///////////////////////////////////////////////////////////////////
 
-///////////////////////////////////////////////////////////////////
-//
-//
-//	METHOD NAME : librpmDb::librpmDb
-//	METHOD TYPE : Constructor
-//
-//	DESCRIPTION :
-//
 librpmDb::librpmDb( const Pathname & root_r, const Pathname & dbPath_r, bool readonly_r )
-    : _d( * new D( root_r, dbPath_r, readonly_r ) )
+: _d( * new D( root_r, dbPath_r, readonly_r ) )
 {}
 
-///////////////////////////////////////////////////////////////////
-//
-//
-//	METHOD NAME : librpmDb::~librpmDb
-//	METHOD TYPE : Destructor
-//
-//	DESCRIPTION :
-//
 librpmDb::~librpmDb()
 {
+  MIL << "Close database " << *this << endl;
   delete &_d;
 }
 
-///////////////////////////////////////////////////////////////////
-//
-//
-//	METHOD NAME : librpmDb::unref_to
-//	METHOD TYPE : void
-//
 void librpmDb::unref_to( unsigned refCount_r ) const
-{
-  if ( refCount_r == 1 )
-  {
-    dbRelease();
-  }
-}
+{ return; } // if ( refCount_r == 1 ) { tbd if we hold a static reference as weak ptr. }
 
-///////////////////////////////////////////////////////////////////
-//
-//
-//	METHOD NAME : librpmDb::root
-//	METHOD TYPE : const Pathname &
-//
 const Pathname & librpmDb::root() const
-{
-  return _d._root;
-}
+{ return _d._root; }
 
-///////////////////////////////////////////////////////////////////
-//
-//
-//	METHOD NAME : librpmDb::dbPath
-//	METHOD TYPE : const Pathname &
-//
 const Pathname & librpmDb::dbPath() const
-{
-  return _d._dbPath;
-}
+{ return _d._dbPath; }
 
-///////////////////////////////////////////////////////////////////
-//
-//
-//	METHOD NAME : librpmDb::error
-//	METHOD TYPE : PMError
-//
-shared_ptr<RpmException> librpmDb::error() const
-{
-  return _d._error;
-}
-
-///////////////////////////////////////////////////////////////////
-//
-//
-//	METHOD NAME : librpmDb::empty
-//	METHOD TYPE : bool
-//
-bool librpmDb::empty() const
-{
-  return( valid() && ! *db_const_iterator( this ) );
-}
-
-///////////////////////////////////////////////////////////////////
-//
-//
-//	METHOD NAME : librpmDb::size
-//	METHOD TYPE : unsigned
-//
-unsigned librpmDb::size() const
-{
-  unsigned count = 0;
-  if ( valid() )
-  {
-    db_const_iterator it( this );
-    for ( db_const_iterator it( this ); *it; ++it )
-      ++count;
-  }
-  return count;
-}
-
-///////////////////////////////////////////////////////////////////
-//
-//
-//	METHOD NAME : librpmDb::dont_call_it
-//	METHOD TYPE : void *
-//
-void * librpmDb::dont_call_it() const
-{
-  return rpmtsGetRdb(_d._ts);
-}
-
-///////////////////////////////////////////////////////////////////
-//
-//
-//	METHOD NAME : librpmDb::dumpOn
-//	METHOD TYPE : ostream &
-//
-//	DESCRIPTION :
-//
 std::ostream & librpmDb::dumpOn( std::ostream & str ) const
-{
-  ReferenceCounted::dumpOn( str ) << _d;
-  return str;
-}
+{ return ReferenceCounted::dumpOn( str ) << _d; }
 
 ///////////////////////////////////////////////////////////////////
 //
 //	CLASS NAME : librpmDb::db_const_iterator::D
-/**
- *
- **/
+//
 class librpmDb::db_const_iterator::D
 {
-  D & operator=( const D & ); // NO ASSIGNMENT!
-  D ( const D & );            // NO COPY!
+  D & operator=( const D & ) = delete; // NO ASSIGNMENT!
+  D ( const D & ) = delete;            // NO COPY!
+  D(D &&);
+  D &operator=(D &&) = delete;
+
 public:
 
-  librpmDb::constPtr     _dbptr;
-  shared_ptr<RpmException> _dberr;
+  librpmDb::constPtr              _dbptr;
+  AutoDispose<rpmdbMatchIterator> _mi;
+  RpmHeader::constPtr             _hptr;
 
-  RpmHeader::constPtr _hptr;
-  rpmdbMatchIterator   _mi;
+  /** A null iterator */
+  D()
+  {}
 
-  D( librpmDb::constPtr dbptr_r )
-      : _dbptr( dbptr_r )
-      , _mi( 0 )
+  /** Open a specific rpmdb if it exists. */
+  D( const Pathname & root_r, const Pathname & dbPath_r = Pathname() )
   {
-    if ( !_dbptr )
-    {
-      try
-      {
-        librpmDb::dbAccess( _dbptr );
-      }
-      catch (const RpmException & excpt_r)
-      {
-        ZYPP_CAUGHT(excpt_r);
-      }
-      if ( !_dbptr )
-      {
-        WAR << "No database access: " << _dberr << endl;
-      }
+    try {
+      _dbptr = librpmDb::dbOpenIf( root_r, dbPath_r );
     }
-    else
-    {
-      destroy(); // Checks whether _dbptr still valid
+    catch ( const RpmException & excpt_r ) {
+      ZYPP_CAUGHT(excpt_r);
     }
-  }
-
-  ~D()
-  {
-    if ( _mi )
-    {
-      ::rpmdbFreeIterator( _mi );
+    if ( not _dbptr ) {
+      WAR << "No database access: " << dumpPath( root_r, dbPath_r ) << endl;
     }
   }
 
@@ -524,27 +289,17 @@ public:
     destroy();
     if ( ! _dbptr )
       return false;
-    _mi = ::rpmtsInitIterator( _dbptr->_d._ts, rpmTag(rpmtag), keyp, keylen );
+    _mi = AutoDispose<rpmdbMatchIterator>( ::rpmtsInitIterator( _dbptr->_d._ts, rpmTag(rpmtag), keyp, keylen ), ::rpmdbFreeIterator );
     return _mi;
   }
 
   /**
-   * Destroy iterator. Invalidates _dbptr, if database was blocked meanwile.
-   * Always returns false.
+   * Reset iterator. Always returns false.
    **/
   bool destroy()
   {
-    if ( _mi )
-    {
-      _mi = ::rpmdbFreeIterator( _mi );
-      _hptr = 0;
-    }
-    if ( _dbptr && _dbptr->error() )
-    {
-      _dberr = _dbptr->error();
-      WAR << "Lost database access: " << _dberr << endl;
-      _dbptr = 0;
-    }
+    _mi.reset();
+    _hptr.reset();
     return false;
   }
 
@@ -584,7 +339,6 @@ public:
   {
     if ( ! create( RPMDBI_PACKAGES ) )
       return false;
-#warning TESTCASE: rpmdbAppendIterator and (non)sequential access?
 #ifdef RPMFILEITERMAX	// since rpm.4.12
     ::rpmdbAppendIterator( _mi, (const unsigned *)&off_r, 1 );
 #else
@@ -603,7 +357,6 @@ public:
     if ( !_mi )
       return 0;
     int ret = ::rpmdbGetIteratorCount( _mi );
-#warning TESTCASE: rpmdbGetIteratorCount returns 0 on sequential access?
     return( ret ? ret : -1 ); // -1: sequential access
   }
 };
@@ -616,163 +369,66 @@ public:
 //
 ///////////////////////////////////////////////////////////////////
 
-///////////////////////////////////////////////////////////////////
-//
-//
-//	METHOD NAME : librpmDb::db_const_iterator::db_iterator
-//	METHOD TYPE : Constructor
-//
+#if LEGACY(1735)
+// Former ZYPP_API used this as default ctor (dbptr_r == nullptr).
+// (dbptr_r!=nullptr) is not possible because librpmDb is not in ZYPP_API.
 librpmDb::db_const_iterator::db_const_iterator( librpmDb::constPtr dbptr_r )
-    : _d( * new D( dbptr_r ) )
-{
-  findAll();
-}
+: db_const_iterator( "/" )
+{}
+#endif
+
+librpmDb::db_const_iterator::db_const_iterator()
+: _d( * new D( "/" ) )
+{ findAll(); }
+
+librpmDb::db_const_iterator::db_const_iterator( const Pathname & root_r )
+: _d( * new D( root_r ) )
+{ findAll(); }
+
+librpmDb::db_const_iterator::db_const_iterator( const Pathname & root_r, const Pathname & dbPath_r )
+: _d( * new D( root_r, dbPath_r ) )
+{ findAll(); }
+
+librpmDb::db_const_iterator::db_const_iterator( std::nullptr_t )
+: _d( * new D() )
+{}
 
-///////////////////////////////////////////////////////////////////
-//
-//
-//	METHOD NAME : librpmDb::db_const_iterator::~db_const_iterator
-//	METHOD TYPE : Destructor
-//
 librpmDb::db_const_iterator::~db_const_iterator()
-{
-  delete &_d;
-}
+{ delete &_d; }
+
+bool librpmDb::db_const_iterator::hasDB() const
+{ return bool(_d._dbptr); }
 
-///////////////////////////////////////////////////////////////////
-//
-//
-//	METHOD NAME : librpmDb::db_const_iterator::operator++
-//	METHOD TYPE : void
-//
 void librpmDb::db_const_iterator::operator++()
-{
-  _d.advance();
-}
+{ _d.advance(); }
 
-///////////////////////////////////////////////////////////////////
-//
-//
-//	METHOD NAME : librpmDb::db_const_iterator::dbHdrNum
-//	METHOD TYPE : unsigned
-//
 unsigned librpmDb::db_const_iterator::dbHdrNum() const
-{
-  return _d.offset();
-}
+{ return _d.offset(); }
 
-///////////////////////////////////////////////////////////////////
-//
-//
-//	METHOD NAME : librpmDb::db_const_iterator::operator*
-//	METHOD TYPE : const RpmHeader::constPtr &
-//
 const RpmHeader::constPtr & librpmDb::db_const_iterator::operator*() const
-{
-  return _d._hptr;
-}
+{ return _d._hptr; }
 
-///////////////////////////////////////////////////////////////////
-//
-//
-//	METHOD NAME : librpmDb::db_const_iterator::dbError
-//	METHOD TYPE : PMError
-//
-shared_ptr<RpmException> librpmDb::db_const_iterator::dbError() const
-{
-  if ( _d._dbptr )
-    return _d._dbptr->error();
-
-  return _d._dberr;
-}
-
-/******************************************************************
-**
-**
-**	FUNCTION NAME : operator<<
-**	FUNCTION TYPE : ostream &
-*/
 std::ostream & operator<<( std::ostream & str, const librpmDb::db_const_iterator & obj )
-{
-  str << "db_const_iterator(" << obj._d._dbptr
-  << " Size:" << obj._d.size()
-  << " HdrNum:" << obj._d.offset()
-  << ")";
-  return str;
-}
+{ return str << "db_const_iterator(" << obj._d._dbptr << ")"; }
 
-///////////////////////////////////////////////////////////////////
-//
-//
-//	METHOD NAME : librpmDb::db_const_iterator::findAll
-//	METHOD TYPE : bool
-//
 bool librpmDb::db_const_iterator::findAll()
-{
-  return _d.init( RPMDBI_PACKAGES );
-}
+{ return _d.init( RPMDBI_PACKAGES ); }
 
-///////////////////////////////////////////////////////////////////
-//
-//
-//	METHOD NAME : librpmDb::db_const_iterator::findByFile
-//	METHOD TYPE : bool
-//
 bool librpmDb::db_const_iterator::findByFile( const std::string & file_r )
-{
-  return _d.init( RPMTAG_BASENAMES, file_r.c_str() );
-}
+{ return _d.init( RPMTAG_BASENAMES, file_r.c_str() ); }
 
-///////////////////////////////////////////////////////////////////
-//
-//
-//	METHOD NAME : librpmDb::db_const_iterator::findByProvides
-//	METHOD TYPE : bool
-//
 bool librpmDb::db_const_iterator::findByProvides( const std::string & tag_r )
-{
-  return _d.init( RPMTAG_PROVIDENAME, tag_r.c_str() );
-}
+{ return _d.init( RPMTAG_PROVIDENAME, tag_r.c_str() ); }
 
-///////////////////////////////////////////////////////////////////
-//
-//
-//	METHOD NAME : librpmDb::db_const_iterator::findByRequiredBy
-//	METHOD TYPE : bool
-//
 bool librpmDb::db_const_iterator::findByRequiredBy( const std::string & tag_r )
-{
-  return _d.init( RPMTAG_REQUIRENAME, tag_r.c_str() );
-}
+{ return _d.init( RPMTAG_REQUIRENAME, tag_r.c_str() ); }
 
-///////////////////////////////////////////////////////////////////
-//
-//
-//	METHOD NAME : librpmDb::db_const_iterator::findByConflicts
-//	METHOD TYPE : bool
-//
 bool librpmDb::db_const_iterator::findByConflicts( const std::string & tag_r )
-{
-  return _d.init( RPMTAG_CONFLICTNAME, tag_r.c_str() );
-}
+{ return _d.init( RPMTAG_CONFLICTNAME, tag_r.c_str() ); }
 
-///////////////////////////////////////////////////////////////////
-//
-//
-//	METHOD NAME : librpmDb::findByName
-//	METHOD TYPE : bool
-//
 bool librpmDb::db_const_iterator::findByName( const std::string & name_r )
-{
-  return _d.init( RPMTAG_NAME, name_r.c_str() );
-}
+{ return _d.init( RPMTAG_NAME, name_r.c_str() ); }
 
-///////////////////////////////////////////////////////////////////
-//
-//
-//	METHOD NAME : librpmDb::db_const_iterator::findPackage
-//	METHOD TYPE : bool
-//
 bool librpmDb::db_const_iterator::findPackage( const std::string & name_r )
 {
   if ( ! _d.init( RPMTAG_NAME, name_r.c_str() ) )
@@ -796,12 +452,6 @@ bool librpmDb::db_const_iterator::findPackage( const std::string & name_r )
   return _d.set( match );
 }
 
-///////////////////////////////////////////////////////////////////
-//
-//
-//	METHOD NAME : librpmDb::db_const_iterator::findPackage
-//	METHOD TYPE : bool
-//
 bool librpmDb::db_const_iterator::findPackage( const std::string & name_r, const Edition & ed_r )
 {
   if ( ! _d.init( RPMTAG_NAME, name_r.c_str() ) )
@@ -819,12 +469,6 @@ bool librpmDb::db_const_iterator::findPackage( const std::string & name_r, const
   return _d.destroy();
 }
 
-///////////////////////////////////////////////////////////////////
-//
-//
-//	METHOD NAME : librpmDb::db_const_iterator::findPackage
-//	METHOD TYPE : bool
-//
 bool librpmDb::db_const_iterator::findPackage( const Package::constPtr & which_r )
 {
   if ( ! which_r )
